//////////////////////////////////////////////////////////
// Motoclock bsw v.1.0
//////////////////////////////////////////////////////////

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>

//////////////////////////////////////////////////////////
// Port defines
//////////////////////////////////////////////////////////

// Port A
#define TCP   0
#define AKU   1
#define TMP   2
#define P2    3
#define E3    4
#define D3    5
#define C3    6
#define A2    7

// Port B
#define C2    0
#define F2    1
#define G2    2
#define D2    3
#define E2    4
#define ONE   5
#define KEY   6
#define MINUS 7

// Port C
#define SCL   0
#define SDA   1
#define B3    2
#define A3    3
#define G3    4
#define F3    5
#define P3    6
#define B2    7

// Port D
#define D4    0
#define C4    1
#define G4    2
#define E4    3
#define B4    4
#define A4    5
#define F4    6
#define P4    7

//Const defines
#define TEMP 1.97
#define VOLT 4.56
#define TMCP 1.61

#define MODE_TIME 0
#define MODE_TEMP 1
#define MODE_TMCP 2
#define MODE_VOLT 3
#define MAX_MODE MODE_VOLT

#define SETUP_TEMP 1
#define SETUP_TMCP 2
#define SETUP_VOLT 3
#define SETUP_HOUR 4
#define SETUP_DMIN 5
#define SETUP_JMIN 6
#define SETUP_SAVE 7
#define SETUP_END 8

#define ACK 1  
#define NOACK 0  

#define DISPWAIT 3920
#define DISPTIME DISPWAIT/2
#define DISPSETUP DISPWAIT/3

#define MODEWAIT 4
#define MODESTARTWAIT 5

#define ALM_MIN_VOLT 116
#define ALM_MAX_VOLT 126
#define ALM_DEF_VOLT 120

#define ALM_MIN_TEMP 0
#define ALM_MAX_TEMP 5
#define ALM_DEF_TEMP 3

#define ALM_MIN_TMCP 90
#define ALM_MAX_TMCP 195
#define ALM_DEF_TMCP 120

#define ALM_TEMP 1
#define ALM_TMCP 2
#define ALM_VOLT 3

//Defines of display Signs
PROGMEM const uint8_t g_D2A[11]=
{
  _BV(A2), //0
        0, //1
  _BV(A2), //2
  _BV(A2), //3
        0, //4
  _BV(A2), //5
  _BV(A2), //6
  _BV(A2), //7
  _BV(A2), //8
  _BV(A2), //9
  0,       //10
};
PROGMEM const uint8_t g_D2B[11]=
{
  _BV(C2) | _BV(D2) | _BV(E2) | _BV(F2),           //0
  _BV(C2),                                         //1
            _BV(D2) | _BV(E2) |           _BV(G2), //2
  _BV(C2) | _BV(D2) |                     _BV(G2), //3
  _BV(C2) |                     _BV(F2) | _BV(G2), //4
  _BV(C2) | _BV(D2) |           _BV(F2) | _BV(G2), //5
  _BV(C2) | _BV(D2) | _BV(E2) | _BV(F2) | _BV(G2), //6
  _BV(C2),                                         //7
  _BV(C2) | _BV(D2) | _BV(E2) | _BV(F2) | _BV(G2), //8
  _BV(C2) | _BV(D2) |           _BV(F2) | _BV(G2), //9
  0,                                               //10
};
PROGMEM const uint8_t g_D2C[11]=
{
  _BV(B2), //0
  _BV(B2), //1
  _BV(B2), //2
  _BV(B2), //3
  _BV(B2), //4
        0, //5
        0, //6
  _BV(B2), //7
  _BV(B2), //8
  _BV(B2), //9
  0,       //10
};
PROGMEM const uint8_t g_D3A[11]=
{
  _BV(C3) | _BV(D3) | _BV(E3), //0
  _BV(C3)                    , //1
            _BV(D3) | _BV(E3), //2
  _BV(C3) | _BV(D3),           //3
  _BV(C3),                     //4
  _BV(C3) | _BV(D3),           //5
  _BV(C3) | _BV(D3) | _BV(E3), //6
  _BV(C3),                     //7  
  _BV(C3) | _BV(D3) | _BV(E3), //8
  _BV(C3) | _BV(D3),           //9
  0,                           //10
 };
PROGMEM const uint8_t g_D3C[11]=
{
  _BV(A3) | _BV(B3) | _BV(F3),           //0
            _BV(B3),                     //1
  _BV(A3) | _BV(B3) |           _BV(G3), //2
  _BV(A3) | _BV(B3) |           _BV(G3), //3
            _BV(B3) | _BV(F3) | _BV(G3), //4
  _BV(A3) |           _BV(F3) | _BV(G3), //5
  _BV(A3) |           _BV(F3) | _BV(G3), //6
  _BV(A3) | _BV(B3),                     //7
  _BV(A3) | _BV(B3) | _BV(F3) | _BV(G3), //8
  _BV(A3) | _BV(B3) | _BV(F3) | _BV(G3), //9
  0,                                     //10  
};
PROGMEM const uint8_t g_D4D[14]=
{
  _BV(A4) | _BV(B4) | _BV(C4) | _BV(D4) | _BV(E4) | _BV(F4),           //0
            _BV(B4) | _BV(C4),                                         //1
  _BV(A4) | _BV(B4) |           _BV(D4) | _BV(E4) |           _BV(G4), //2
  _BV(A4) | _BV(B4) | _BV(C4) | _BV(D4) |                     _BV(G4), //3
            _BV(B4) | _BV(C4) |                     _BV(F4) | _BV(G4), //4
  _BV(A4) |           _BV(C4) | _BV(D4) |           _BV(F4) | _BV(G4), //5
  _BV(A4) |           _BV(C4) | _BV(D4) | _BV(E4) | _BV(F4) | _BV(G4), //6
  _BV(A4) | _BV(B4) | _BV(C4),                                         //7
  _BV(A4) | _BV(B4) | _BV(C4) | _BV(D4) | _BV(E4) | _BV(F4) | _BV(G4), //8
  _BV(A4) | _BV(B4) | _BV(C4) | _BV(D4) |           _BV(F4) | _BV(G4), //9
  0,                                                                   //10
                      _BV(C4) | _BV(D4) | _BV(E4),                     //11 v
  _BV(A4) |                                         _BV(F4) | _BV(G4), //12 c
  _BV(A4) |           _BV(D4) |                               _BV(G4), //13 s
};

//////////////////////////////////////////////////////////
// DISPLAY
//////////////////////////////////////////////////////////

void ClearDisp()
{
  PORTA |= _BV(A2) | _BV(C3) | _BV(D3) | _BV(E3) | _BV(P2);
  PORTB |= _BV(C2) | _BV(D2) | _BV(E2) | _BV(F2) | _BV(G2) | _BV(ONE) | _BV(MINUS) | _BV(KEY);
  PORTC |= _BV(A3) | _BV(B3) | _BV(F3) | _BV(G3) | _BV(B2) | _BV(P3);
  PORTD |= _BV(A4) | _BV(B4) | _BV(C4) | _BV(D4) | _BV(E4) | _BV(F4) | _BV(G4) | _BV(P4);
}	

//display number value
void DispNumber(uint16_t val)
{
  uint8_t disp4 = val%10;
  val = val/10;
  uint8_t disp3 = val%10;
  val = val/10;
  uint8_t disp2 = val%10;
  val = val/10;
  uint8_t disp1 = val%10;
  
  ClearDisp();
  
  PORTA &= ~ (pgm_read_word(&g_D2A[disp2]) | pgm_read_word(&g_D3A[disp3]));
  PORTB &= ~ (((disp1>0) ? _BV(ONE) : 0) | pgm_read_word(&g_D2B[disp2]));
  PORTC &= ~ (pgm_read_word(&g_D2C[disp2]) | pgm_read_word(&g_D3C[disp3])); 
  PORTD &= ~ pgm_read_word(&g_D4D[disp4]);

}

//Display temperature
void DispTemp(int16_t tmp, uint8_t mode, uint8_t blink) //0-temp 1-Tmpc  //1-setup 2-alarm
{
  ClearDisp();
  if(2==blink) return;

  uint8_t symb = 12;
  if (mode>0) symb = 13;
  PORTD &= ~ (pgm_read_word(&g_D4D[symb]) | _BV(P4));
  if(1==blink) return;

  uint8_t sign = 0;
  if (tmp<0) 
  {
     sign = _BV(MINUS);
     tmp = -tmp;
  }
  if (tmp>99) 
  {
     sign = _BV(ONE);
  }
  uint8_t disp2 = tmp%10;
  uint8_t disp1 = 10;
  if (tmp>9)
  {
     tmp = tmp/10;
     disp1 = tmp%10;
  }   

  PORTA &= ~ (pgm_read_word(&g_D2A[disp1]) | pgm_read_word(&g_D3A[disp2]));
  PORTB &= ~ (pgm_read_word(&g_D2B[disp1]) | sign);
  PORTC &= ~ (pgm_read_word(&g_D2C[disp1]) | pgm_read_word(&g_D3C[disp2]));
}

//Display voltage
void DispVolt(uint8_t nap, uint8_t blink) //1-setup 2-alarm
{
  ClearDisp();
  if(2==blink) return;
  
  PORTD &= ~ (pgm_read_word(&g_D4D[11]));
  if(1==blink) return;

  int8_t one = 0;
  if (nap>99) 
  {
     one = _BV(ONE);
  }
  uint8_t disp2 = nap%10;
  nap = nap/10;
  uint8_t disp1 = nap%10;
 
  PORTA &= ~ (pgm_read_word(&g_D2A[disp1]) | pgm_read_word(&g_D3A[disp2]) | _BV(P2));
  PORTB &= ~ (pgm_read_word(&g_D2B[disp1]) | one);
  PORTC &= ~ (pgm_read_word(&g_D2C[disp1]) | pgm_read_word(&g_D3C[disp2])); 
}

//Display Time
void DispTime(uint8_t hour, uint8_t min, uint8_t blink) //1-time separator 2-hour 3-decmin 4-min
{
  uint8_t dwd = 0;
  uint8_t dwg = 0;
  
  if(blink!=1)
  {
	 dwd = _BV(P2);
	 dwg = _BV(P3); 
  }	  
  
  uint8_t one = 0;
  if (hour>12) hour=hour-12;
  if (hour>9) one = _BV(ONE);
  uint8_t disp2 = hour%10;

  uint8_t disp4 = min%10;
  min = min/10;
  uint8_t disp3 = min%10;

  if(2==blink)
  {
	 one=0;
	 disp2=10;  
  }	  

  if(3==blink)
  {
	 disp3=10; 
  }	 
  
  if(4==blink)
  {
	 disp4=10;  	 
  }	  

  ClearDisp();

  PORTA &= ~ (pgm_read_word(&g_D2A[disp2]) | pgm_read_word(&g_D3A[disp3]) | dwd);
  PORTB &= ~ (pgm_read_word(&g_D2B[disp2]) | one);
  PORTC &= ~ (pgm_read_word(&g_D2C[disp2]) | pgm_read_word(&g_D3C[disp3]) | dwg); 
  PORTD &= ~ pgm_read_word(&g_D4D[disp4]);
}

//////////////////////////////////////////////////////////
// TWI
//////////////////////////////////////////////////////////
 
void twiStart(void)  
{  
  TWCR = _BV(TWINT) | _BV(TWSTA) | _BV(TWEN);  
  while (!(TWCR & _BV(TWINT))); 
}  
  
void twiStop(void)  
{  
  TWCR = _BV(TWINT) | _BV(TWEN) | _BV(TWSTO);  
  while ((TWCR & _BV(TWSTO)));  
}  
  
void twiWrite(char data)  
{  
  TWDR = data;  
  TWCR = _BV(TWINT) | _BV(TWEN);  
  while (!(TWCR & _BV(TWINT)));  
}  
  
char twiRead(char ack)  
{  
  TWCR = _BV(TWINT) | _BV(TWEN) | (ack<<TWEA); 
  while (!(TWCR & _BV(TWINT)));  
  return TWDR;  
}  
   
//////////////////////////////////////////////////////////
// CLOCK
//////////////////////////////////////////////////////////
     
volatile uint8_t g_Hour = 12;
volatile uint8_t g_Min = 0;
      
uint8_t dec2bcd(uint8_t dec)
{  
   return ((dec / 10)<<4) | (dec % 10);  
}  
  
uint8_t bcd2dec(uint8_t bcd)
{  
   return ((((bcd) >> 4) & 0x0F) * 10) + ((bcd) & 0x0F);  
}  

void ReadTime()
{
   uint8_t min=0,hour=0;  

   twiStart();  
   twiWrite(0xA2); //RTC adress
   twiWrite(0x02);  

   twiStart();  
   twiWrite(0xA3);  
   
   min  = twiRead(ACK);   //sec
   min  = twiRead(ACK);   //min 
   hour = twiRead(NOACK); //hour
   twiStop();  
   
   hour = bcd2dec(hour & 0x3f);
   
   if (hour>12) hour=hour-12;
   if (hour==0) hour=12;
  
   g_Hour = hour; 
   g_Min = bcd2dec(min & 0x7f);
}

void SetTime(uint8_t hour, uint8_t min)
{
   twiStart();  
   twiWrite(0xA2); //RTC adress   
   twiWrite(0x02);  
   twiWrite(0x00); //writing sec
   uint8_t buf = dec2bcd(min);
   twiWrite(buf);  //writing min
   buf = dec2bcd(hour);
   twiWrite(buf);  //writing hour
   twiStop(); 
}

//////////////////////////////////////////////////////////
// MAIN
//////////////////////////////////////////////////////////
#define EEPROM __attribute__ ((section (".eeprom")))

uint8_t eeprom_Volt EEPROM;
uint8_t eeprom_Temp EEPROM;
uint8_t eeprom_Tmcp EEPROM;

volatile uint8_t g_Msr_mode = MODE_VOLT;
volatile int16_t g_MsrTemp,g_MsrVolt, g_MsrTmcp;
volatile int16_t g_Temp,g_Volt,g_Tmcp;

volatile uint8_t g_AlmTemp,g_AlmVolt,g_AlmTmcp;

volatile uint8_t g_SetupVal,g_SetupMin,g_SetupSec;

volatile uint16_t g_DispWait = 0;
volatile uint8_t g_Dark = 0;
volatile uint8_t g_ModeWait = MODESTARTWAIT;

volatile uint8_t g_Mode = MODE_VOLT;
volatile uint8_t g_SetupMode = 0;
volatile int8_t g_AlmMode = 0;

int main(void)
{
    //JTAG interface Off
    MCUCSR = _BV(JTD);
    MCUCSR = _BV(JTD);

    //Input ports
    DDRA = _BV(P2) | _BV(E3) | _BV(D3) | _BV(C3) | _BV(A2);
    DDRB = _BV(C2) | _BV(F2) | _BV(G2) | _BV(D2) | _BV(E2) | _BV(MINUS) | _BV(ONE);
    DDRC = _BV(B3) | _BV(A3) | _BV(G3) | _BV(F3) | _BV(P3) | _BV(B2);
    DDRD = _BV(D4) | _BV(C4) | _BV(G4) | _BV(E4) | _BV(B4) | _BV(A4) | _BV(F4) | _BV(P4);

    //A/C setup
    ADMUX = _BV(REFS0) | _BV(REFS1); //VREF=Internal
    ADMUX = (ADMUX & 0xF8) | _BV(MUX0); //ADC1 - VOLT
    
    ADCSRA |= _BV(ADPS1) | _BV(ADPS2);  //prescaler/64
    ADCSRA |= _BV(ADIE); //Generate interrupt on measure end 
    ADCSRA |= _BV(ADEN); //set on ADC
    ADCSRA |= _BV(ADSC); //start conversion
   
    //Timera 0 - display service
    TIMSK = _BV(TOIE0);  //T/C service
    TCCR0 = _BV(CS01);   //prescaler/8
   
    // Timera 1 - clock service
    TIMSK |= _BV(TOIE1); //T/C service
    TCCR1A = 0x00;
    TCCR1B = _BV(CS12);  //prescaler/256

    uint8_t btn = 0;
   
    //Reading eeprom settings
    
    btn = eeprom_read_byte(&eeprom_Temp);
    if((btn<ALM_MIN_TEMP)||(btn>ALM_MAX_TEMP)) btn = ALM_DEF_TEMP;  
    g_AlmTemp = btn;

    btn = eeprom_read_byte(&eeprom_Volt);
    if((btn<ALM_MIN_VOLT)||(btn>ALM_MAX_VOLT)) btn = ALM_DEF_VOLT;  
    g_AlmVolt = btn;

    btn = eeprom_read_byte(&eeprom_Tmcp);
    if((btn<ALM_MIN_TMCP)||(btn>ALM_MAX_TMCP)||(btn%5!=0)) btn = ALM_DEF_TMCP;  
    g_AlmTmcp = btn;
   
    g_Volt=255;
   
    //global interrupts
    sei();

    ClearDisp();
    _delay_ms(10); //wait 4 btn cap
   
    for(;;)
    {
	    btn = 0; 

        if(!(PINB & _BV(KEY))) //btn was pushed
        {
            _delay_ms(5);
            btn = 1; 
            //btn still pushed
            while (!(PINB & _BV(KEY)))
            {
               g_ModeWait = MODEWAIT;
               _delay_ms(10);

               if(btn>0) btn++;
               
               //btn long pushed
               if(btn>40) 
               {
				    switch(g_SetupMode)
		            {
					    case SETUP_TEMP:
					       g_SetupMode=SETUP_SAVE;
					       g_AlmTemp = g_SetupVal;
					       eeprom_write_byte(&eeprom_Temp,g_AlmTemp);
					       _delay_ms(150);
				           g_SetupMode=SETUP_END;
				        break;
				        case SETUP_TMCP:
					       g_SetupMode=SETUP_SAVE;
					       g_AlmTmcp = g_SetupVal;
					       eeprom_write_byte(&eeprom_Tmcp,g_AlmTmcp);
					       _delay_ms(150);
				           g_SetupMode=SETUP_END;
				        break;  
				        case SETUP_VOLT:
					       g_SetupMode=SETUP_SAVE;
					       g_AlmVolt = g_SetupVal;
					       eeprom_write_byte(&eeprom_Volt,g_AlmVolt);
					       _delay_ms(150);
				           g_SetupMode=SETUP_END;
				        break; 
				        case SETUP_HOUR:
				        case SETUP_DMIN:
				           g_SetupMode++;
				        break;
				        case SETUP_JMIN:
				           g_SetupMode=SETUP_SAVE;
				           SetTime(g_SetupVal,10*g_SetupMin+g_SetupSec);
				           g_Hour = g_SetupVal;
				           g_Min = 10*g_SetupMin+g_SetupSec;
				           _delay_ms(150);
				           g_SetupMode=SETUP_END;
				        break;
				        default:
				           if(g_Dark>0) g_Dark=0; 
				                   else g_Dark=1;
				    } 
                    btn=0;
			   }	                  
            }
            
            //btn present
            if(btn>0)
            {
                switch(g_SetupMode)
	            {
				    case SETUP_TEMP:
			           if(g_SetupVal<ALM_MAX_TEMP) g_SetupVal++; else g_SetupVal=ALM_MIN_TEMP;
			        break; 
				    case SETUP_TMCP:
			           if(g_SetupVal<ALM_MAX_TMCP) g_SetupVal+=5; else g_SetupVal=ALM_MIN_TMCP;
			        break; 
				    case SETUP_VOLT:
			           if(g_SetupVal<ALM_MAX_VOLT) g_SetupVal++; else g_SetupVal=ALM_MIN_VOLT;
			        break; 
				    case SETUP_HOUR:
			           if(g_SetupVal<12) g_SetupVal++; else g_SetupVal=1;
			        break;
			        case SETUP_DMIN:
			           if(g_SetupMin<5) g_SetupMin++; else g_SetupMin=0;
			        break;
			        case SETUP_JMIN:
			           if(g_SetupSec<9) g_SetupSec++; else g_SetupSec=0;
			        break;
			        default:
			            if(g_AlmMode>0) g_AlmMode=-g_AlmMode;
			            else
 			            if(g_Mode<MAX_MODE) g_Mode++; else g_Mode = 0;
 			                                    
			    } 
            }   
	    }  
    }	   
	   
    return 0;
}

//////////////////////////////////////////////////////////
// INTERRUPT
//////////////////////////////////////////////////////////

 
// Measures
ISR(ADC_vect)
{
   int16_t Msr = ADC;
   float fMsr = Msr;
   
   switch(g_Msr_mode)
   {
      case MODE_TEMP: //temp
         g_MsrTemp = fMsr / TEMP - 274;
         g_Msr_mode++;
         ADMUX = (ADMUX & 0xF8); //ADC0 - TMCP
      break; 
      case MODE_TMCP: //tmcp
         g_MsrTmcp = fMsr / TMCP + g_Temp;
         g_Msr_mode++;
         ADMUX = (ADMUX & 0xF8) | _BV(MUX0); //ADC1 - VOLT
      break; 
      case MODE_VOLT: //volt 
         g_MsrVolt = fMsr / VOLT;
         //only in start mode:
         if(g_Volt>200) g_Volt = g_MsrVolt;

         //jump into setup mode - when setup btn connected to ground
         if((0==g_SetupMode)&&(g_MsrVolt<10))
         {
			g_ModeWait = MODEWAIT;
			switch(g_Mode)
            {
		        case MODE_TIME:
		           g_SetupVal = g_Hour;
		           g_SetupSec = g_Min%10;
		           g_SetupMin = g_Min/10;
                   g_SetupMin = g_SetupMin%10;
		           g_SetupMode = SETUP_HOUR;
		        break;
		        case MODE_TEMP:
		           g_SetupVal = g_AlmTemp;
		           g_SetupMode = SETUP_TEMP;
		        break; 	  
			    case MODE_VOLT:
		           g_SetupVal = g_AlmVolt;
		           g_SetupMode = SETUP_VOLT;
		        break; 
			    case MODE_TMCP:
		           g_SetupVal = g_AlmTmcp;
		           g_SetupMode = SETUP_TMCP;
		        break; 
		    } 
	     }
         //return from setup mode
         if((g_SetupMode>=SETUP_END)&&(g_MsrVolt>10)) g_SetupMode = 0;

         g_Msr_mode=MODE_TEMP;
         ADMUX = (ADMUX & 0xF8) | _BV(MUX1); //ADC2 - TEMP
      break; 
     
   }   

   //start conversion
   ADCSRA |= _BV(ADSC);
}

// Display
ISR(TIMER0_OVF_vect)
{
   uint8_t blink = 0;
   g_DispWait++;
   
   if(0==g_DispWait%2) 
   {
      if(g_Dark>0) ClearDisp();
   }
   else   
   {
        if((g_SetupMode>0)&&(g_SetupMode<SETUP_END)) //setup mods
        {
            if(((g_DispWait>DISPTIME)?(g_DispWait-DISPTIME):g_DispWait)>DISPSETUP)  blink=1; else blink=0; 
           
            switch(g_SetupMode)
            {
			    case SETUP_TEMP:
		           DispTemp(g_SetupVal,0,blink);
		        break; 
		        case SETUP_TMCP:
		           DispTemp(g_SetupVal,1,blink);
		        break;
			    case SETUP_VOLT:
			       DispVolt(g_SetupVal,blink);
		        break; 
			     
		        case SETUP_HOUR:
		        case SETUP_DMIN:
		        case SETUP_JMIN:
		           DispTime(g_SetupVal,g_SetupMin*10+g_SetupSec,blink*(g_SetupMode-SETUP_HOUR+2));
		        break;
		        case SETUP_SAVE:
		           ClearDisp();   
		    }            
        }
        else
        if(g_AlmMode>0) //alarm mods
        {
		    if(((g_DispWait>DISPTIME)?(g_DispWait-DISPTIME):g_DispWait)>DISPSETUP)  blink=2; else blink=0; 	
		    
            switch(g_AlmMode) 
			    {
				    case ALM_TEMP:
			           DispTemp(g_Temp,0,blink);
			        break; 
				    case ALM_VOLT:
			           DispVolt(g_Volt,blink);
			        break; 
				    case ALM_TMCP:
			           DispTemp(g_Tmcp,1,blink); 
			        break; 
			    }
	    }				    
        else switch(g_Mode) //normal mods
           {
		        case MODE_TIME:
		           if(g_DispWait>DISPTIME) blink=0; else blink=1;
		           DispTime(g_Hour,g_Min,blink);
		        break;	  
			    case MODE_TEMP:
		           DispTemp(g_Temp,0,0);
		        break;
		        case MODE_TMCP:
		           DispTemp(g_Tmcp,1,0); 
		        break; 
			    case MODE_VOLT:
		           DispVolt(g_Volt,0);
		        break; 
			    
		    }
   }  
   
   //Reset counter
   if(g_DispWait>=DISPWAIT) g_DispWait=0;
}

int8_t mabs(int8_t val)
{
   if(val<0) return -val; else return val;
}	

////Reading time & rewrite measures, setting alarms - every ~ 2 sec
ISR(TIMER1_OVF_vect)
{
   ReadTime();

   if(g_ModeWait>0) g_ModeWait--;
   else {
	        if (g_SetupMode>0) g_SetupMode=SETUP_END;
	        if (g_Mode>0) g_Mode=0;
	    }   

   g_Temp = g_MsrTemp;
   g_Tmcp = g_MsrTmcp;
   g_Volt = g_MsrVolt;
   
   if(g_ModeWait>0) return;
     
   if((g_Volt<=g_AlmVolt)&&(g_AlmMode==0)) g_AlmMode = ALM_VOLT;
   if((g_Volt>g_AlmVolt)&&(g_AlmMode==mabs(ALM_VOLT))) g_AlmMode = 0;
   
   if((g_Temp<=g_AlmTemp)&&(g_AlmMode==0)) g_AlmMode = ALM_TEMP;
   if((g_Temp>g_AlmTemp)&&(g_AlmMode==mabs(ALM_TEMP))) g_AlmMode = 0;
   
   if((g_Tmcp>=g_AlmTmcp)&&(g_AlmMode==0)) g_AlmMode = ALM_TMCP;
   if((g_Tmcp<g_AlmTmcp)&&(g_AlmMode==mabs(ALM_TMCP))) g_AlmMode = 0;
   
}

//////////////////////////////////////////////////////////
// END
//////////////////////////////////////////////////////////
